;
;               STE DMASOUND INTERFACING for GFABASIC
;               (c) 2004 by Simon Sunnyboy / Paradize
;               V2.0
;
;  ChangeLog 6.6.2004
;  - initiale Version
;  - unsigned-> signed Konvertierung
;  - play normal und loop
;


snd_cntl_DMA    EQU $ff8901
snd_start_hi_DMA EQU $ff8903
snd_start_mid_DMA EQU $ff8905
snd_start_lo_DMA EQU $ff8907
snd_end_hi_DMA  EQU $ff890f
snd_end_mid_DMA EQU $ff8911
snd_end_lo_DMA  EQU $ff8913
snd_mode_DMA    EQU $ff8921


; jump table

; +0
                bra     play_once
; +4
                bra     play_loop
; +8
                bra     stop_sound
; +12
                bra     convert_sample
                EVEN
                DC.B "STE DMASOUND Interfacing V2.0",13,10
                DC.B "(c) 2004 by Simon Sunnyboy / Paradize",0
                EVEN
; --------------
; convert_sample
; --------------
; convert unsigned STFM samples into signed STE format
; The sample is converted in place.
; inputs:
; a0 : pointer to sample start
; a1 : pointer to sample end (STE convention)
;
convert_sample:
; save d0
                move.l  d0,-(sp)
convert_loop:
; read sample byte
                move.b  (a0),d0
; to signed
                sub.b   #$80,d0
; write back
                move.b  d0,(a0)+
; end of sample?
                cmpa.l  a1,a0
                bne.s   convert_loop
; restore d0
                move.l  (sp)+,d0
                rts

; ---------
; play_once
; ---------
;
; play given sample once
;
; inputs:
; a0 - start of sample
; a1 - end of sample
; d0 - snd_mode according to STE spec (rate and format)
;
play_once:
                move.l  d0,-(sp) ; save d0
; stop DMA
                clr.b   snd_cntl_DMA

; set sample format
                move.b  d0,snd_mode_DMA

; set start sample location
                move.l  a0,d0
                move.b  d0,snd_start_lo_DMA
                lsr.l   #$08,d0
                move.b  d0,snd_start_mid_DMA
                lsr.l   #$08,d0
                move.b  d0,snd_start_hi_DMA

; set end sample location
                move.l  a1,d0
                move.b  d0,snd_end_lo_DMA
                lsr.l   #$08,d0
                move.b  d0,snd_end_mid_DMA
                lsr.l   #$08,d0
                move.b  d0,snd_end_hi_DMA

; play DMA sound once
                move.b  #$01,snd_cntl_DMA

                move.l  (sp)+,d0 ; restore d0
                rts
; ---------
; play_loop
; ---------
;
; play given sample looping
;
; inputs:
; a0 - start of sample
; a1 - end of sample
; d0 - snd_mode according to STE spec (rate and format)
;
play_loop:
                move.l  d0,-(sp) ; save d0

; stop DMA
                clr.b   snd_cntl_DMA

; set snd_mode
                move.b  d0,snd_mode_DMA

; set start sample location
                move.l  a0,d0
                move.b  d0,snd_start_lo_DMA
                lsr.l   #$08,d0
                move.b  d0,snd_start_mid_DMA
                lsr.l   #$08,d0
                move.b  d0,snd_start_hi_DMA

; set end sample location
                move.l  a1,d0
                move.b  d0,snd_end_lo_DMA
                lsr.l   #$08,d0
                move.b  d0,snd_end_mid_DMA
                lsr.l   #$08,d0
                move.b  d0,snd_end_hi_DMA

; start DMA sound in loop mode
                move.b  #$03,snd_cntl_DMA
                move.l  (sp)+,d0
                rts

; stops a running DMA sound
stop_sound:     clr.b   snd_cntl_DMA
                rts

                END
